@@ -52,16 +52,22 @@
 #include "eog-collection-marshal.h"
 #include "eog-full-screen.h"
 #include "eog-info-view.h"
+#include "eog-transform.h"
+#include "eog-save-dialog.h"
+#include "eog-horizontal-splitter.h"
+#include "eog-vertical-splitter.h"
 
 enum {
 	PROP_WINDOW_TITLE,
 	PROP_WINDOW_STATUS,
+	PROP_IMAGE_PROGRESS,
 	PROP_LAST
 };
 
 static const gchar *property_name[] = {
 	"window/title", 
-	"window/status"
+	"window/status",
+	"image/progress"
 };
 
 enum {
@@ -85,6 +91,7 @@ struct _EogCollectionViewPrivate {
 
 	BonoboPropertyBag       *property_bag;
 	BonoboZoomable          *zoomable;
+	BonoboControl           *control;
 
 	BonoboUIComponent       *uic;
 
@@ -95,6 +102,7 @@ struct _EogCollectionViewPrivate {
 	gboolean                need_update_prop[PROP_LAST];
 
 	EogImage                *displayed_image;
+	guint                   progress_handler_id;
 
 	gboolean                has_zoomable_frame;
 };
@@ -166,6 +174,347 @@ verb_ImageNext_cb (BonoboUIComponent *uic,
 	eog_wrap_list_select_right (EOG_WRAP_LIST (view->priv->wraplist));
 }
 
+static void 
+verb_Undo_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	GList *images;
+	GList *it;
+	EogCollectionView *view;
+
+	view = EOG_COLLECTION_VIEW (data);
+
+	images = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (view->priv->wraplist));	
+
+	for (it = images; it != NULL; it = it->next) {
+		EogImage *image = EOG_IMAGE (it->data);
+		eog_image_undo (image);
+	}
+}
+
+/*
+ * Image saving stuff.
+ *
+ */
+
+typedef struct {
+	EogCollectionView *view;
+	GList             *image_list;
+	GtkWidget         *dlg;
+	gboolean          cancel;
+	gulong            finished_id;
+	gulong            failed_id;
+	int               n_images;
+	int               count;
+} SaveContext;
+
+static const char* save_threat_cmds[] = {
+	"/commands/FlipHorizontal",
+	"/commands/FlipVertical",
+        "/commands/Rotate90cw",
+	"/commands/Rotate90ccw",
+	"/commands/Rotate180",
+        "/commands/Undo",
+        "/commands/Save", 
+	"/commands/SlideShow",
+	NULL
+};
+
+static void save_image_init_loading (SaveContext *ctx);
+
+static void
+set_commands_sensitive_state (EogCollectionView *view, const char *command[], gboolean sensitive)
+{
+	int i = 0;
+	
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (view));
+
+	for (i = 0; command[i] != NULL; i++) {
+		bonobo_ui_component_set_prop (view->priv->uic, command[i], "sensitive",
+					      sensitive ? "1" : "0", NULL);
+	}
+}
+
+
+static void
+save_image_loading_failed (EogImage *image, char *message, gpointer data)
+{
+	SaveContext *ctx;
+
+	ctx = (SaveContext *) data;
+
+	g_signal_handler_disconnect (G_OBJECT (image), ctx->finished_id);
+	g_signal_handler_disconnect (G_OBJECT (image), ctx->failed_id);
+	
+	g_object_unref (image);
+
+	save_image_init_loading (ctx);
+}
+
+static void
+save_image_loading_finished (EogImage *image, gpointer data)
+{
+	SaveContext *ctx;
+
+	ctx = (SaveContext *) data;
+
+	g_signal_handler_disconnect (G_OBJECT (image), ctx->finished_id);
+	g_signal_handler_disconnect (G_OBJECT (image), ctx->failed_id);
+
+	if (eog_image_is_modified (image) && !ctx->cancel) {
+			GError *error = NULL;
+			GnomeVFSURI *uri;
+			
+			uri = eog_image_get_uri (image);
+			
+			if (!eog_image_save (image, uri, &error)) {
+				/* FIXME: indicate failed state to the user */
+			}
+			gnome_vfs_uri_unref (uri);
+	}
+
+	if (image != ctx->view->priv->displayed_image) {
+		eog_image_free_mem (image);
+	}
+
+	g_object_unref (image);
+
+	save_image_init_loading (ctx);
+}
+
+
+static void
+save_image_init_loading (SaveContext *ctx)
+{
+	if (ctx->image_list == NULL || ctx->cancel) {
+		if (ctx->cancel) 
+			eog_save_dialog_update (EOG_SAVE_DIALOG (ctx->dlg), (double) ctx->count / (double) ctx->n_images,
+						_("Cancel saving"));
+		else 
+			eog_save_dialog_update (EOG_SAVE_DIALOG (ctx->dlg), 1.0,
+						_("Saving finished"));
+		
+		gtk_dialog_response (GTK_DIALOG (ctx->dlg), GTK_RESPONSE_OK);
+	}
+	else {
+		EogImage *image = EOG_IMAGE (ctx->image_list->data);
+
+		eog_save_dialog_update (EOG_SAVE_DIALOG (ctx->dlg), (double) ctx->count++ / (double) ctx->n_images,
+					eog_image_get_caption (image));
+
+		ctx->finished_id = g_signal_connect (image, "loading_finished", G_CALLBACK (save_image_loading_finished), ctx);
+		ctx->failed_id = g_signal_connect (image, "loading_failed", G_CALLBACK (save_image_loading_failed), ctx);
+		ctx->image_list = g_list_delete_link (ctx->image_list, ctx->image_list);
+
+		g_object_ref (image);
+
+		eog_image_load (image, EOG_IMAGE_LOAD_COMPLETE);
+	}
+
+	while (gtk_events_pending ()) {
+		gtk_main_iteration ();
+	}
+}
+
+static gboolean
+save_dlg_close (gpointer data)
+{
+	gtk_widget_destroy (GTK_WIDGET (data));
+	return FALSE;
+}
+
+static gboolean
+save_dlg_delete_cb (GtkWidget *dialog,
+		    GdkEvent *event,
+		    gpointer user_data)
+{
+	SaveContext *ctx;
+
+	ctx = (SaveContext*) user_data;
+	ctx->cancel = TRUE;
+
+	return TRUE;
+}
+
+static void
+save_dlg_response_cb (GtkDialog *dialog,
+		      gint response_id,
+		      gpointer user_data)
+{
+	SaveContext *ctx;
+	EogCollectionView *view;
+
+	ctx = (SaveContext*) user_data;
+
+	switch (response_id) {
+	case GTK_RESPONSE_OK:
+		view = ctx->view;
+		if (ctx->image_list != NULL) {
+			g_list_free (ctx->image_list);
+		}
+		g_free (ctx);
+		g_timeout_add (1250, save_dlg_close, dialog);
+
+		/* reenable commands that may disturb the save state */
+		set_commands_sensitive_state (view, save_threat_cmds, TRUE);
+
+		break;
+
+	default:
+		ctx->cancel = TRUE;
+	}
+}
+
+static void 
+verb_Save_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	EogCollectionView *view;
+	EogCollectionViewPrivate *priv;
+	SaveContext *ctx;
+
+	view = EOG_COLLECTION_VIEW (data);
+	priv = view->priv;
+
+	ctx = g_new0 (SaveContext, 1);
+	ctx->image_list = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (priv->wraplist));
+	ctx->view = view;
+	ctx->cancel = FALSE;
+
+	if (ctx->image_list == NULL) {
+		g_free (ctx);
+		return;
+	}
+
+	ctx->n_images = g_list_length (ctx->image_list);
+	ctx->count = 0;
+
+	/* disable commands that may disturb the save state */
+	set_commands_sensitive_state (view, save_threat_cmds, FALSE);
+
+	ctx->dlg = eog_save_dialog_new ();
+
+	bonobo_control_set_transient_for (BONOBO_CONTROL (priv->control), GTK_WINDOW (ctx->dlg), NULL);
+	g_signal_connect (G_OBJECT (ctx->dlg), "delete-event", G_CALLBACK (save_dlg_delete_cb), ctx);
+	g_signal_connect (G_OBJECT (ctx->dlg), "response", G_CALLBACK (save_dlg_response_cb), ctx);
+
+	gtk_widget_show_now (GTK_WIDGET (ctx->dlg));
+
+	save_image_init_loading (ctx);
+}
+
+
+/*
+ *  Image transforming stuff.
+ *
+ */
+
+typedef struct {
+	EogCollectionView *view;
+	float max_progress;
+	int counter;
+} ProgressData;
+
+static void
+image_progress_cb (EogImage *image, float progress, gpointer data)
+{
+	ProgressData *pd;
+	float total;
+	BonoboArg *arg;
+	EogCollectionViewPrivate *priv;
+
+	if (EOG_IS_COLLECTION_VIEW (data)) {
+		pd = NULL;
+		priv = EOG_COLLECTION_VIEW (data)->priv;
+	}
+	else {
+		pd = (ProgressData*) data;
+		priv = pd->view->priv;
+	}
+
+	if (!pd) {
+		total = progress;
+	}
+	else {
+		total = ((float) pd->counter + progress) / pd->max_progress;
+	}
+
+	arg = bonobo_arg_new (BONOBO_ARG_FLOAT);
+	BONOBO_ARG_SET_FLOAT (arg, total);
+
+	bonobo_event_source_notify_listeners (priv->property_bag->es,
+					      "image/progress",
+					      arg, NULL);
+	bonobo_arg_release (arg);
+}
+
+static void 
+apply_transformation (EogCollectionView *view, EogTransform *trans)
+{
+	GList *images;
+	GList *it;
+	gint id;
+	ProgressData *data;
+
+	images = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (view->priv->wraplist));	
+	data = g_new0 (ProgressData, 1);
+	data->view = view;
+	data->max_progress = g_list_length (images);
+	data->counter = 0;
+
+	for (it = images; it != NULL; it = it->next) {
+		EogImage *image = EOG_IMAGE (it->data);
+
+		id = g_signal_connect (G_OBJECT (image), "progress", (GCallback) image_progress_cb, data);
+
+		eog_image_transform (image, trans);
+
+		g_signal_handler_disconnect (image, id);
+		data->counter++;
+	}
+
+	g_free (data);
+	g_object_unref (trans);
+}
+
+static void
+verb_FlipHorizontal_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_flip_new (EOG_TRANSFORM_FLIP_HORIZONTAL));
+}
+
+static void
+verb_FlipVertical_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_flip_new (EOG_TRANSFORM_FLIP_VERTICAL));
+}
+
+static void
+verb_Rotate90ccw_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_rotate_new (270));
+}
+
+static void
+verb_Rotate90cw_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_rotate_new (90));
+}
+
+static void
+verb_Rotate180_cb (BonoboUIComponent *uic, gpointer data, const char *name)
+{
+	g_return_if_fail (EOG_IS_COLLECTION_VIEW (data));
+
+	apply_transformation (EOG_COLLECTION_VIEW (data), eog_transform_rotate_new (180));
+}
+
 static void
 verb_ZoomIn_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
@@ -228,6 +577,13 @@ static BonoboUIVerb collection_verbs[] = {
 	BONOBO_UI_VERB ("SlideShow", verb_SlideShow_cb),
 	BONOBO_UI_VERB ("ImageNext", verb_ImageNext_cb),
 	BONOBO_UI_VERB ("ImagePrev", verb_ImagePrev_cb),
+	BONOBO_UI_VERB ("FlipHorizontal",verb_FlipHorizontal_cb),
+	BONOBO_UI_VERB ("FlipVertical",  verb_FlipVertical_cb),
+	BONOBO_UI_VERB ("Rotate90cw",    verb_Rotate90cw_cb),
+	BONOBO_UI_VERB ("Rotate90ccw",   verb_Rotate90ccw_cb),
+	BONOBO_UI_VERB ("Rotate180",     verb_Rotate180_cb),
+	BONOBO_UI_VERB ("Undo",          verb_Undo_cb),
+	BONOBO_UI_VERB ("Save",          verb_Save_cb),
 	BONOBO_UI_VERB_END
 };
 
@@ -241,9 +597,11 @@ eog_collection_view_create_ui (EogCollectionView *view)
 	priv = view->priv;
 
 	/* Set up the UI from an XML file. */
-        bonobo_ui_util_set_ui (priv->uic, DATADIR,
-			       "eog-collection-view-ui.xml", "EogCollectionView", NULL);
+	bonobo_ui_util_set_ui (priv->uic, DATADIR,
+						   "eog-collection-view-ui.xml", "EogCollectionView", NULL);
 
+	bonobo_ui_component_set_prop (priv->uic, "/menu/Edit/Eog EditPreferences Separator",
+								  "hidden", "0", NULL);
 	bonobo_ui_component_add_verb_list_with_data (priv->uic,
 						     collection_verbs,
 						     view);
@@ -554,17 +912,24 @@ handle_selection_changed (EogWrapList *list, EogCollectionView *view)
 
 	priv = view->priv;
 
-	if (priv->displayed_image != NULL) {
-		eog_image_free_mem (priv->displayed_image);
-		g_object_unref (priv->displayed_image);
-		priv->displayed_image = NULL;
+	if (priv->displayed_image != image) {
+		if (priv->displayed_image != NULL) {
+			g_signal_handler_disconnect (priv->displayed_image, priv->progress_handler_id);
+
+			eog_image_free_mem (priv->displayed_image);
+			g_object_unref (priv->displayed_image);
+			priv->displayed_image = NULL;
+		}
+		
+		priv->progress_handler_id = g_signal_connect (image, "progress", G_CALLBACK (image_progress_cb), view);
+		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->scroll_view), image);
+		if (priv->info_view != NULL) {
+			eog_info_view_set_image (EOG_INFO_VIEW (priv->info_view), image);
+		}
+		
+		priv->displayed_image = image;
 	}
-
-	eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->scroll_view), image);
-	eog_info_view_set_image (EOG_INFO_VIEW (priv->info_view), image);
-
-	priv->displayed_image = image;
-	
+		
 	update_status_text (view);
 }
 
@@ -747,8 +1112,6 @@ load_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 
 	}
 
-	g_print ("load_uri_cb: %s\n", (char*) text_uri);
-
 	eog_collection_model_add_uri (priv->model, (gchar*)text_uri); 
 
 	return 0;
@@ -775,6 +1138,7 @@ create_user_interface (EogCollectionView *list_view)
 	frame = gtk_widget_new (GTK_TYPE_FRAME, "shadow-type", GTK_SHADOW_IN, NULL);
 	gtk_container_add (GTK_CONTAINER (frame), priv->scroll_view);
 
+#if HAVE_EXIF
 	/* info view for additional image information */
 	priv->info_view = gtk_widget_new (EOG_TYPE_INFO_VIEW, NULL);
 	sw = gtk_scrolled_window_new (NULL, NULL);
@@ -782,16 +1146,16 @@ create_user_interface (EogCollectionView *list_view)
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (sw), priv->info_view);
+#else
+	priv->info_view = NULL;
+	sw = NULL;
+#endif
 
-	hpaned = gtk_hpaned_new ();
+	hpaned = gtk_hpaned_new (); /* FIXME: use specalised eog splitter */
 	gtk_paned_pack1 (GTK_PANED (hpaned), frame, TRUE, TRUE);
 	gtk_paned_pack2 (GTK_PANED (hpaned), sw, FALSE, TRUE);
 	gtk_widget_show_all (hpaned);
 
-#if !HAVE_EXIF
-	gtk_paned_set_position (GTK_PANED (hpaned), 10000);
-#endif
-
 	/* the wrap list for all the thumbnails */
 	priv->wraplist = eog_wrap_list_new ();
 	g_object_set (G_OBJECT (priv->wraplist), 
@@ -805,11 +1169,14 @@ create_user_interface (EogCollectionView *list_view)
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+					GTK_POLICY_AUTOMATIC,
+					GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (sw), priv->wraplist);
 	gtk_widget_show_all (sw);
 
 	/* put it all together */
-	vpaned = gtk_vpaned_new ();
+	vpaned = eog_vertical_splitter_new ();
 	
 	gtk_paned_pack1 (GTK_PANED (vpaned), hpaned, TRUE, TRUE);
 	gtk_paned_pack2 (GTK_PANED (vpaned), sw, TRUE, TRUE);
@@ -845,6 +1212,7 @@ eog_collection_view_construct (EogCollectionView *list_view)
 
 	/* interface Bonobo::Control */
 	control = bonobo_control_new (root);
+	priv->control = control;
 	g_signal_connect (control, "activate", G_CALLBACK (control_activate_cb), list_view);
 
 	bonobo_object_add_interface (BONOBO_OBJECT (list_view),
@@ -901,6 +1269,9 @@ eog_collection_view_construct (EogCollectionView *list_view)
 	bonobo_property_bag_add (priv->property_bag, property_name[PROP_WINDOW_STATUS], PROP_WINDOW_STATUS,
 				 BONOBO_ARG_STRING, NULL, _("Status Text"),
 				 BONOBO_PROPERTY_READABLE);
+	bonobo_property_bag_add (priv->property_bag, "image/progress", PROP_IMAGE_PROGRESS,
+				 BONOBO_ARG_FLOAT, NULL, _("Progress of Image Loading"),
+				 BONOBO_PROPERTY_READABLE);
 	bonobo_control_set_properties (BONOBO_CONTROL (control), 
 				       BONOBO_OBJREF (priv->property_bag),
 				       NULL);
@@ -911,8 +1282,6 @@ eog_collection_view_construct (EogCollectionView *list_view)
 	/* UI Component */
 	priv->uic = bonobo_ui_component_new ("EogCollectionView");
 
-	bonobo_object_dump_interfaces (BONOBO_OBJECT (list_view));
-
 	return list_view;
 }
 
