@@ -4,6 +4,7 @@
 #include <glib-object.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include "eog-transform.h"
 
 G_BEGIN_DECLS
 
@@ -49,6 +50,7 @@ struct _EogImageClass {
 	void (* loading_failed) (EogImage *img, const char* message);
 	void (* loading_cancelled) (EogImage *img);
 	void (* loading_info_finished) (EogImage *img);
+	void (* progress) (EogImage *img, float progress);
 	
 	void (* thumbnail_finished) (EogImage *img);
 	void (* thumbnail_failed) (EogImage *img);
@@ -83,13 +85,11 @@ gboolean            eog_image_is_modified                     (EogImage *img);
 gchar*              eog_image_get_caption                     (EogImage *img);
 const gchar*        eog_image_get_collate_key                 (EogImage *img);
 gpointer            eog_image_get_exif_information            (EogImage *img);
+GnomeVFSURI*        eog_image_get_uri                         (EogImage *img);
 
 /* modification API */
-void                eog_image_rotate_clock_wise               (EogImage *img);
-void                eog_image_rotate_counter_clock_wise       (EogImage *img);
-void                eog_image_rotate_180                      (EogImage *img);
-void                eog_image_flip_horizontal                 (EogImage *img);
-void                eog_image_flip_vertical                   (EogImage *img);
+void                eog_image_transform                       (EogImage *img, EogTransform *trans);
+void                eog_image_undo                            (EogImage *img);
 
 G_END_DECLS
 
